﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using CarTrackr.Data;
using CarTrackr.Domain;
using CarTrackr.Controllers;

namespace CarTrackr.Repository
{
    public class CarRepository : IDataBoundRepository, ICarRepository
    {
        #region Constructor

        public CarRepository(CarTrackrData dataSource)
        {
            DataSource = dataSource;
        }

        #endregion

        #region IDataBoundRepository Members

        public CarTrackrData DataSource { get; set; }

        #endregion

        #region ICarRepository Members

        public User User { get; set; }

        public List<Car> List()
        {
            return DataSource.DataContext.Cars.Where(c => c.OwnerId == User.RowKey).ToList();
        }

        Car ICarRepository.RetrieveById(Guid id)
        {
            return DataSource.DataContext.Cars.Where(c => c.RowKey == id.ToString()).SingleOrDefault();
        }

        public Car RetrieveByLicensePlate(string licensePlate)
        {
            return DataSource.DataContext.Cars.Where(c => c.OwnerId == User.RowKey && c.LicensePlate == licensePlate).SingleOrDefault();
        }

        public void Add(Car car)
        {
            if (car.OwnerId == Guid.Empty.ToString() || string.IsNullOrEmpty(car.OwnerId))
                car.OwnerId = User.RowKey;

            car.EnsureValid();

            DataSource.DataContext.AddObject("Cars", car);
            DataSource.DataContext.SaveChanges();
        }

        public void Update(Car car)
        {
            car.EnsureValid();

            DataSource.DataContext.UpdateObject(car);
            DataSource.DataContext.SaveChanges();
        }

        #endregion
    }
}